###############################################################################
# SoundFX.py -- Sound Effect Player
# Copyright 2004, Chris Gonnerman
# All Rights Reserved
# 
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions
# are met:
# 
# Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer. 
# 
# Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution. 
# 
# Neither the name of the author nor the names of any contributors
# may be used to endorse or promote products derived from this software
# without specific prior written permission. 
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 
###############################################################################

import pygame
from pygame.locals import *

import os, glob, string, sys
import time

j = os.path.join

###############################################################################
#  variable initialization
###############################################################################

class generic:
    pass

btnlist = []
btn_letters = [ ]
tab_letters = [ ]

curtab = None

done = None

### Initialize colors ###

WHITE = (255, 255, 255)
BLACK = (0, 0, 0)
GREY = (128, 128, 128)
NEOBLACK = (90, 90, 90)
RED = (255, 0, 0)
GREEN = (0, 255, 0)
BLUE = (0, 0, 255)
YELLOW = (255, 255, 0)
PURPLE = (255, 0, 255)
ORANGE = (255, 128, 64)
BROWN = (172, 112, 0)
TRANSGREY = (128, 128, 128, 64)

col_ofs = 70
row_ofs = 27

btn_width = 80
btn_xgap = 3
btn_xofs = 11
btn_xcount = 7

btn_height = 77
btn_ygap = 3
btn_yofs = 9
btn_ycount = 6

btn_pagecount = btn_ycount * btn_xcount

curchannel = None
cursound = None

lowers = "abcdefghijklmnopqrstuvwxyz"
uppers = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
volkeys = "`1234567890"

###############################################################################
#  class and function definitions
###############################################################################

def vpupdate():
    pygame.display.update(viewport)

class TButton:
    def __init__(self, rect, key, callback = None, obj = None):
        self.rect = rect
        self.key = key
        self.__callback = callback
        self.obj = obj
        btnlist.append(self)
    def claim(self, point):
        if self.rect.collidepoint(point):
            self.callback(point)
            return self
        return None
    def callback(self, point = None):
        if self.__callback is not None:
            self.__callback(self, point)

class Drawable:
    def __init__(self, pic, pos, bg = None):
        self.pic = pic
        self.pos = pos
        self.bg = bg
        self.rect = None
        self.blank = 0
        self.draw()
    def draw(self, bg = None, update = 0, blank = None):
        if bg is not None:
            self.bg = bg
        if blank is not None:
            self.blank = blank
        if self.rect is None:
            self.rect = scr.blit(self.pic, self.pos)
        if self.bg is not None:
            scr.fill(self.bg, self.rect)
        if not self.blank:
            scr.blit(self.pic, self.pos)
        if update:
            pygame.display.update(self.rect)
        return self.rect

def clickhandler(point):
    for b in btnlist:
        if b.claim(point):
            return b
    return None

### Construct the tab dictionary and keylist ###

tabdict = {}

names = os.listdir("sounds")
names.sort()
names = names[:22]

for n in names:
    tabdict[n] = generic()
    tabdict[n].buttons = os.listdir(j("sounds", n))
    tabdict[n].buttons.sort()
    tabdict[n].buttons = tabdict[n].buttons[:btn_pagecount]

tablist = tabdict.keys()
tablist.sort()
tablist = tablist[:22]

tabpos = [
    (0, 1, 0),
    (0, 2, 0),
    (0, 3, 0),
    (0, 4, 0),
    (0, 5, 0),
    (0, 6, 0),

    (1, 7, 1),
    (2, 7, 1),
    (3, 7, 1),
    (4, 7, 1),
    (5, 7, 1),

    (6, 6, 2),
    (6, 5, 2),
    (6, 4, 2),
    (6, 3, 2),
    (6, 2, 2),
    (6, 1, 2),

    (5, 0, 3),
    (4, 0, 3),
    (3, 0, 3),
    (2, 0, 3),
    (1, 0, 3),
]

for i in range(len(tablist)):

    tab = tabdict[tablist[i]]
    tab.index = i
    tab.pos = tabpos[i]
    tab.text = tablist[i]

    row, col, orient = tab.pos

    if orient == 0 or orient == 2:
        col = col_ofs + ((col-1) * 100)
        if row == 0:
            row = row_ofs
        else:
            row = row_ofs + 500
    else:
        row = row_ofs + ((row-1) * 100)
        if col == 0:
            col = col_ofs
        else:
            col = col_ofs + 600

    tab.rowcol = (row, col, orient)

    if orient == 0:
        tab.rect = pygame.Rect((col + 1, row - 22, 99, 22))
    elif orient == 1:
        tab.rect = pygame.Rect((col, row + 1, 22, 99))
    elif orient == 2:
        tab.rect = pygame.Rect((col + 1, row, 99, 22))
    elif orient == 3:
        tab.rect = pygame.Rect((col - 22, row + 1, 22, 99))

def drawonetab(tab):
    global curtab

    curtab = tabdict[tablist[tab]]
    
    row, col, orient = curtab.rowcol

    # draw the frame

    scr.fill(BLACK, (col_ofs - 2, row_ofs - 2, 604, 504))
    scr.fill(GREY, (col_ofs, row_ofs, 600, 500))

    txt_img = FONT.render(curtab.text, 1, WHITE, GREY)
    ltr_img = tab_letters[tab]

    # draw the tab

    if orient == 0:
        pygame.draw.line(scr, BLACK, (col - 1, row - 1), (col - 1, row - 22), 2)
        pygame.draw.line(scr, BLACK, (col + 99, row - 22), (col + 99, row - 1), 2)

        scr.fill(GREY, (col + 1, row - 22, 98, 22))

        scr.blit(ltr_img, (col + 96 - ltr_img.get_width(), row - 20))
        scr.blit(txt_img, (col + 8, row - 20), (0, 0, 98, 20))

    elif orient == 1:
        pygame.draw.line(scr, BLACK, (col + 1, row - 1), (col + 22, row - 1), 2)
        pygame.draw.line(scr, BLACK, (col + 22, row + 100), (col + 1, row + 100), 2)

        scr.fill(GREY, (col, row + 1, 22, 99))

        ltr_img = pygame.transform.rotate(ltr_img, -90)
        scr.blit(ltr_img, (col + 2, row + 96 - ltr_img.get_height()))

        txt_img = pygame.transform.rotate(txt_img, -90)
        scr.blit(txt_img, (col + 2, row + 8), (0, 0, 20, 98))

    elif orient == 2:
        pygame.draw.line(scr, BLACK, (col-1, row + 2), (col - 1, row + 23), 2)
        pygame.draw.line(scr, BLACK, (col + 99, row + 23), (col + 99, row + 2), 2)

        scr.fill(GREY, (col + 1, row, 98, 22))

        scr.blit(ltr_img, (col + 96 - ltr_img.get_width(), row + 3))
        scr.blit(txt_img, (col + 8, row + 3), (0, 0, 98, 20))

    elif orient == 3:
        pygame.draw.line(scr, BLACK, (col - 1, row - 1), (col - 22, row - 1), 2)
        pygame.draw.line(scr, BLACK, (col - 22, row + 100), (col - 1, row + 100), 2)

        scr.fill(GREY, (col - 22, row + 1, 22, 99))

        ltr_img = pygame.transform.rotate(ltr_img, 90)
        scr.blit(ltr_img, (col - 20, row + 8))

        txt_img = pygame.transform.rotate(txt_img, 90)
        scr.blit(txt_img, (col - 20, row + 96 - txt_img.get_height()), (0, 0, 20, 98))

    # draw the buttons

    for i in range(len(curtab.buttons)):
        brow = int(i / btn_xcount)
        bcol = i - (brow * btn_xcount)
        rect = (col_ofs + btn_xofs + (bcol * (btn_width + btn_xgap)) + 2, 
                row_ofs + btn_yofs + (brow * (btn_height + btn_ygap)) + 2,
                btn_width - 4, btn_height - 4)
        scr.fill(BLACK, (rect[0]-2,rect[1]-2,rect[2]+4,rect[3]+4))
        scr.fill(GREY, rect)
        # handle the shortcut
        if i < len(btn_letters):
            ltr = btn_letters[i][0]
            if btn_letters[i][1] is None:
                btn_letters[i][1] = \
                    (rect[0] + btn_width - ltr.get_width() - 8,
                     rect[1] + btn_height - ltr.get_height() - 1)
            scr.blit(ltr, btn_letters[i][1])
        txt = curtab.buttons[i]
        if txt[-4:].lower() == ".wav":
            txt = txt[:-4]
        txt_img = BTNFONT.render(txt, 1, BLACK)
        # wrap the button text if needed
        words = string.split(txt, " ")
        if txt_img.get_width() > (btn_width - 8) and len(words) > 1:
            lines = []
            curline = ""
            for w in words:
                img = BTNFONT.render(string.strip(curline + " " + w), 1, BLACK)
                if img.get_width() > (btn_width - 8):
                    if curline:
                        lines.append(curline)
                        curline = w
                    else:
                        lines.append(w)
                        curline = ""
                else:
                    curline = string.strip(curline + " " + w)
            if curline:
                lines.append(curline)
            r = 0
            for l in lines:
                txt_img = BTNFONT.render(l, 1, BLACK)
                scr.blit(txt_img, (rect[0] + 2, rect[1]+5+r), (0,0,btn_width-8,btn_height-8))
                r += 12
        else:
            scr.blit(txt_img, (rect[0] + 2, rect[1]+5), (0,0,btn_width-8,btn_height-8))

def drawtab(pos = -1):
    if pos == -1: # draw them all
        for l in uppers:
            tab_letters.append(FONT.render(l, 1, NEOBLACK))
        for i in range(len(tablist) - 1, -1, -1):
            drawonetab(i)
    else:
        drawonetab(pos)

def tabhandler(tab_btn, point):
    drawtab(tab_btn.obj.index)

for i in range(len(tablist)):
    tab = tabdict[tablist[i]]
    tab.button = TButton(tab.rect, uppers[i], tabhandler, tab)

###############################################################################
#  button initialization
###############################################################################

def buttonhandler(btn, point):
    global curchannel, cursound
    try:
        name = j("sounds", curtab.text, curtab.buttons[btn.obj])
    except:
        return
    if curchannel is not None:
        curchannel.stop()
    cursound = pygame.mixer.Sound(name)
    cursound.set_volume(volume / 10.0)
    if curchannel is None:
        curchannel = pygame.mixer.Channel(0)
    curchannel.set_volume(1.0)
    curchannel.play(cursound)

def setupbuttons():
    # buttons
    for col in range(btn_xcount):
        for row in range(btn_ycount):
            l = row * btn_ycount + col
            letter = None
            if l < len(lowers):
                letter = lowers[l]
            TButton(pygame.Rect(
                    col_ofs + btn_xofs + (col * (btn_width + btn_xgap)), 
                    row_ofs + btn_yofs + (row * (btn_height + btn_ygap)),
                    btn_width, btn_height),
                letter, buttonhandler, row * btn_xcount + col)
    # button shortcut letters
    for l in lowers:
        btn_letters.append([BIGFONT.render(l, 1, NEOBLACK), None])

###############################################################################
#  volume control
###############################################################################

volume = 5

vol_xpos = 13
vol_ypos = 174
vol_interval = 25
vol_height = 20
vol_width = 20
vol_extraht = 78

def framevolume():
    scr.fill(GREY, \
        (vol_xpos - 5, vol_ypos - 5, vol_width + 10, (vol_interval * 11) + 5 + vol_extraht))
    scr.fill(BLACK, \
        (vol_xpos - 3, vol_ypos - 3, vol_width + 6, (vol_interval * 11) + 1 + vol_extraht))
    txt_img = FONT.render("Volume", 1, WHITE)
    txt_img = pygame.transform.rotate(txt_img, 90)
    scr.blit(txt_img, (vol_xpos + 1, vol_ypos + (vol_interval * 11) + 9))

def drawvolume():
    v = volume / 10.0
    for i in range(11):
        r = (vol_xpos, (i * vol_interval) + vol_ypos, vol_width, vol_height)
        TButton(pygame.Rect(r), volkeys[i], volumehandler, i)
        if i == volume:
            scr.fill(GREEN, r)
        else:
            scr.fill(GREY, r)
        txt_img = BTNFONT.render("%02d" % i, 1, BLACK)
        scr.blit(txt_img, (vol_xpos + 4, (i * vol_interval) + vol_ypos + 3), (0, 0, 21, 21))

def volumehandler(btn, point):
    global volume
    volume = btn.obj
    drawvolume()
    if cursound is not None:
        cursound.set_volume(volume / 10.0)

###############################################################################
#  misc buttons
###############################################################################

stop_xpos = 13
stop_ypos = 103
stop_width = 20
stop_height = 45

def stophandler(btn, point):
    if curchannel is not None:
        curchannel.stop()

def drawstop():
    global stop_xpos, stop_ypos, stop_width, stop_height
    txt_img = FONT.render("Stop", 1, WHITE)
    txt_img = pygame.transform.rotate(txt_img, 90)
    stop_rect = txt_img.get_rect()
    txt_height = stop_rect[3]
    stop_rect = pygame.Rect(stop_xpos - 5, stop_ypos - 5, stop_width + 10, stop_height + 19)
    scr.fill(GREY, stop_rect)
    scr.fill(RED, (stop_rect[0] + 2, stop_rect[1] + 2, stop_rect[2] - 4, stop_rect[3] - 4))
    scr.blit(txt_img, (stop_xpos + 1, int(stop_ypos + 3 + ((stop_height - txt_height) / 2))))
    TButton(stop_rect, '\033', stophandler, None)

quit_xpos = 13
quit_ypos = 33
quit_width = 20
quit_height = 45

def quithandler(btn, point):
    pass

def drawquit():
    global quit_xpos, quit_ypos, quit_width, quit_height
    txt_img = FONT.render("Quit", 1, YELLOW)
    txt_img = pygame.transform.rotate(txt_img, 90)
    quit_rect = txt_img.get_rect()
    txt_height = quit_rect[3]
    quit_rect = pygame.Rect(quit_xpos - 5, quit_ypos - 5, quit_width + 10, quit_height + 19)
    scr.fill(GREY, quit_rect)
    scr.fill(RED, (quit_rect[0] + 2, quit_rect[1] + 2, quit_rect[2] - 4, quit_rect[3] - 4))
    scr.blit(txt_img, (quit_xpos + 1, int(quit_ypos + 6 + ((quit_height - txt_height) / 2))))
    TButton(quit_rect, None, quithandler, None)

###############################################################################
#  keyboard assistant
###############################################################################

def process_key(ev):
    if ev.unicode:
        return ev.unicode.encode("utf8")
    return ev.key

###############################################################################
#  pygame initialization
###############################################################################

pygame.init()

pygame.display.set_caption("SoundFX by Solomoriah V1.0")

icon = pygame.image.load("speaker.png")
pygame.display.set_icon(icon)

scr = pygame.display.set_mode((700,560), pygame.HWSURFACE|pygame.DOUBLEBUF, 16)

pygame.mouse.set_visible(1)

BIGFONT = pygame.font.Font("resources/freesansbold.ttf", 18)
FONT = pygame.font.Font("resources/freesansbold.ttf", 14)
BTNFONT = pygame.font.Font("resources/freesansbold.ttf", 10)

setupbuttons()

framevolume()
drawvolume()

drawtab(-1)

drawstop()
drawquit()

pygame.display.update()

###############################################################################
#  Main Body
###############################################################################

while not done:
    k = pygame.event.wait()
    if k.type == pygame.QUIT:
        done = 1
    elif k.type == pygame.KEYDOWN:
        rk = process_key(k)
        for b in btnlist:
            if rk == b.key:
                b.callback()
                break
    elif k.type == pygame.MOUSEBUTTONDOWN and k.button == 1:
        b = clickhandler(k.pos)
    pygame.display.update()


###############################################################################
# end of file.
###############################################################################
